# -*- encoding : utf-8 -*-

module Kadmos
  class Importer

    def initialize corpus, options={}
      raise "Corpus has to be given." unless corpus && corpus.is_a?(KadmosGraph)
      @corpus = corpus
      @type_cache = options[:cache_size] ? LruRedux::Cache.new(options[:cache_size]) : ThreadSafe::Cache.new
      @normalized_type_cache = options[:cache_size] ? LruRedux::Cache.new(options[:cache_size]) : ThreadSafe::Cache.new
      reset_counters
    end

    def import &block
      instance_eval &block
      @corpus.commit
    end

    def document args={}
      reset_counters
      @current_token    = nil
      @current_sentence = nil
      @current_line     = nil
      @current_chapter  = nil
      @current_page     = nil
      @current_document = @corpus.get_document_node args
    end

    def author args={}
      raise "No active document." unless @current_document
      existing_author = @corpus.get_author_node(string_value:args[:string_value]) if args[:string_value]
      return existing_author if existing_author
      @corpus.draw_authorship_edge @current_document, new_author = @corpus.get_author_node(args)
      new_author
    end

    def unconnected_author args={}
      @corpus.create_author_node args
    end

    def structural_element type, *parent_types
      new_node = @corpus.create_token_node                                  if type == :token
      new_node = @corpus.create_structural_element_node structure_type:type if type != :token
      old_node = instance_variable_get("@current_#{type}")
      @corpus.draw_next_edge old_node, new_node if old_node
      parent_types.each do |parent_type|
        parent_node = instance_variable_get("@current_#{parent_type}")
        counter_name = "#{parent_type}_#{type}".to_sym
        @corpus.draw_belongs_to_edge new_node, parent_node, in_sequence(counter_name) if parent_node
      end
      instance_variable_set("@current_#{type}", new_node)
    end

    def sentence
      reset_counter :sentence_token
      structural_element :sentence, :chapter, :document
    end

    def chapter
      reset_counter :chapter_sentence
      structural_element :chapter, :document
    end

    def line
      reset_counter :line_token
      structural_element :line, :page, :document
    end

    def page
      reset_counter :page_line
      structural_element :page, :document
    end

    def section description
      reset_counter :section_token
      new_section = structural_element :section, :document
      new_section[:string_value] = description
      new_section
    end

    def normalize string
      string.downcase
    end

    def token string, punctuation_before=nil, punctuation_after=nil
      raise "No active document." unless @current_document
      return if string == ""
      type = @type_cache[string]||=@corpus.find_type_nodes(string_value:string).first
      if !type
        normalized_string = normalize string
        return if normalized_string == ""
        type = @type_cache[string] = @corpus.create_type_node string_value:string
        normalized_type = @normalized_type_cache[normalized_string]||=@corpus.find_normalized_type_nodes(string_value:normalized_string).first
        if !normalized_type
          normalized_type = @normalized_type_cache[normalized_string] = @corpus.create_normalized_type_node string_value:normalized_string
        end
        @corpus.draw_normalization_edge type, normalized_type
      end
      token = structural_element :token, :sentence, :line, :document, :section
      @corpus.draw_type_token_edge type, token
    end

    def in_sequence name
      {sequence_number:@counters[name]+=1}
    end

    def reset_counter name
      @counters[name] = -1
    end
    def reset_counters
      @counters = Hash.new -1
    end

    def commit
      @corpus.commit
    end

  end
end
