# -*- encoding : utf-8 -*-

module Kadmos

  class ::Pacer::PacerGraph
    def export
      buffer = Java::javaIo::ByteArrayOutputStream.new
      Java::comTinkerpopBlueprintsUtilIoGraphml::GraphMLWriter.outputGraph @blueprints_graph, buffer
      buffer.to_string #Java::javaNioCharset::StandardCharsets::UTF_8
    ensure
      buffer.close if buffer
    end
  end

  class KadmosGraph < ::Pacer::PacerGraph

    def initialize input_graph, is_blueprints=false
      @titan_graph = input_graph unless is_blueprints
      @blueprints_graph = is_blueprints ? input_graph : input_graph.blueprints_graph
      path = "kadmos_graph:#{is_blueprints ? @blueprints_graph.to_s : @blueprints_graph.configuration.backend_description}"
      opening = proc { ::Pacer.open_graphs[path] ||= @blueprints_graph }
      closing = proc { @blueprints_graph.shutdown ; ::Pacer.open_graphs.delete[path] }
      super KadmosGraphEncoder, opening, closing
    end

    def titan_graph
      @titan_graph
    end
 
    def commit *args
      yield_result = yield if block_given?
      transaction_result = @blueprints_graph.stop_transaction Java::ComTinkerpopBlueprints::TransactionalGraph::Conclusion::SUCCESS
      yield_result || args || transaction_result || self
    end

    def v_lookup property, values
      values.map{|v| @blueprints_graph.get_vertices(property.to_s,v.to_java).reject{|v|v==nil}.to_a}.to_a.flatten.to_route(:graph=>self,:based_on=>v,:unwrap=>true,:route_name=>"V-IndexMulti")
    end

    module Route
      def mytest
        0
      end
    end

    def string_prefix s
      titan_graph.query{ has('string_value', Java::comThinkaureliusTitanCoreAttribute::Text::PREFIX, s) }
    end

  end

  class KadmosGraphEncoder < ::Pacer::SimpleEncoder

    def self.encode_property(value)
      if value.is_a? String
        value unless value == ''
      elsif false == value
        false.to_java
      else
        value
      end
    end

  end

  module KadmosVertexRouteExtension

    def exact_count
      map{}.count
    end

    def references
      map(&:reference)
    end

    def names
      map(:names).flatten
    end

  end

end

module ::Pacer::Core::Graph::VerticesRoute
  def self.included klass
    klass.class_eval do
      include ::Kadmos::KadmosVertexRouteExtension
    end
  end
end

class ::Pacer::Wrappers::VertexWrapper

  def names
    out(:name).map{|n|n[:string_value]}
  end

  def ensure_name s
    graph.draw_name_edge self, graph.create_name_node(string_value:s) unless names.include? s
  end

  def reference opts={:lang=>:en}
    paths = deepest{|n| n.out_e(:belongs_to).in_v}.paths.to_a
    paths = paths.map{|p|p.map{|x|
      type = x[:structure_type]
      position = x[:sequence_number]
      next type if type
      next position+1 if position
      x
    }}
    document = paths.first.last
    current_type = paths.first.first
    current_type = "token" unless current_type.is_a? String
    paths = paths.map{|p|p[1..-2]}.sort_by{|x|[-x.length,x.select{|y|y.respond_to? :upcase}]}
    case opts[:lang]
    when :de
      "TODO: German reference naming"
    when :en
      "#{paths.map{|p| "#{current_type} #{p.each_slice(2).map{|p,t| t ? "#{p} of #{t}" : "#{p}"}.join(" ")}"}.join(" which is also ")} within #{document.names.first} from #{document.out(:authorship).map{|a|a.names.first}.join(" and ")}".capitalize
     else
      inspect
    end
  end

end